@@ -17,7 +17,7 @@ import pytest
 
 # local
 from .conftest import IS_WINDOWS
-from .accessories import TestTerminal, unicode_cap, as_subprocess
+from .accessories import TestTerminal, unicode_cap, as_subprocess, pty_test
 
 
 def test_export_only_Terminal():
@@ -358,7 +358,7 @@ def test_winsize_IOError_returns_environ():
     @as_subprocess
     def child():
         def side_effect(fd):
-            raise IOError
+            raise OSError
 
         term = TestTerminal()
         term._winsize = side_effect
@@ -399,6 +399,25 @@ def test_yield_hidden_cursor(all_terms):
     child(all_terms)
 
 
+@pytest.mark.skipif(IS_WINDOWS, reason="windows lacks disable_line_wrap capability")
+# see https://github.com/Rockhopper-Technologies/jinxed/blob/main/jinxed/terminfo/vtwin10.py
+# "Removed - These do not appear to be supported" for rmam + smam
+def test_yield_no_line_wrap():
+    """Ensure ``no_line_wrap()`` writes disable and enable VT100 line wrap sequence."""
+    @as_subprocess
+    def child():
+        t = TestTerminal(stream=StringIO(), force_styling=True)
+        with t.no_line_wrap():
+            pass
+        result = t.stream.getvalue()
+        assert t.disable_line_wrap and t.enable_line_wrap
+        assert result == t.disable_line_wrap + t.enable_line_wrap
+        assert result == t.rmam + t.smam
+        assert result == '\x1b[?7l' + '\x1b[?7h'
+
+    child()
+
+
 @pytest.mark.skipif(IS_WINDOWS, reason="windows doesn't work like this")
 def test_no_preferredencoding_fallback():
     """Ensure empty preferredencoding value defaults to ascii."""
@@ -571,3 +590,40 @@ def test_query_methods_respect_does_styling_and_is_a_tty(force_styling, is_a_tty
         assert stream.getvalue() == ''
 
     child()
+
+
+@pytest.mark.skipif(IS_WINDOWS, reason="PTY tests not supported on Windows")
+def test_scroll_region_context_manager():
+    """Test scroll_region context manager sets and resets scroll region."""
+    def child(term):
+        stream = io.StringIO()
+        term._stream = stream
+        with term.scroll_region(top=5, height=10):
+            pass
+        return stream.getvalue()
+
+    output = pty_test(child, rows=24, cols=80)
+    assert output == '\x1b[6;15r\x1b[1;24r'
+
+
+@pytest.mark.skipif(IS_WINDOWS, reason="PTY tests not supported on Windows")
+def test_scroll_region_context_manager_defaults():
+    """Test scroll_region context manager with default arguments."""
+    def child(term):
+        stream = io.StringIO()
+        term._stream = stream
+        with term.scroll_region():
+            pass
+        return stream.getvalue()
+
+    output = pty_test(child, rows=24, cols=80)
+    assert output == '\x1b[1;24r\x1b[1;24r'
+
+
+def test_get_fgcolor_bgcolor_invalid_bits():
+    """Test get_fg/bgcolor raises ValueError for invalid bits parameter."""
+    term = TestTerminal(stream=StringIO())
+    with pytest.raises(ValueError, match=r"bits must be 8 or 16, got 24"):
+        term.get_fgcolor(bits=24)
+    with pytest.raises(ValueError, match=r"bits must be 8 or 16, got 32"):
+        term.get_bgcolor(bits=32)
